@@ -20,17 +20,17 @@ from typing import List
 
 from isofit.utils import segment, extractions, empirical_line
 from isofit.core import isofit, common
+from isofit.core.common import envi_header
 
 EPS = 1e-6
 CHUNKSIZE = 256
-SEGMENTATION_SIZE = 400
 
 UNCORRELATED_RADIOMETRIC_UNCERTAINTY = 0.01
 
 INVERSION_WINDOWS = [[380.0, 1340.0], [1450, 1800.0], [1970.0, 2500.0]]
 
 
-def main():
+def main(rawargs=None):
     """ This is a helper script to apply OE over a flightline using the MODTRAN radiative transfer engine.
 
     The goal is to run isofit in a fairly 'standard' way, accounting for the types of variation that might be
@@ -75,6 +75,9 @@ def main():
             at fine scale resolution.  Choices - 0 off, 1 on.  Default 0
         ray_temp_dir (Optional, str): Location of temporary directory for ray parallelization engine.  Default is
             '/tmp/ray'
+        emulator_base (Optional, str): Location of emulator base path.  Point this at the base of sRTMnet to use the
+            emulator instead of MODTRAN.
+        segmentation_size (Optional, int): Size of segments to construct for empirical line (if used).
 
             Reference:
             D.R. Thompson, A. Braverman,P.G. Brodrick, A. Candela, N. Carbon, R.N. Clark,D. Connelly, R.O. Green, R.F.
@@ -82,6 +85,11 @@ def main():
             D.S. Wettergreen. Quantifying Uncertainty for Remote Spectroscopy of Surface Composition. Remote Sensing of
             Environment, 2020. doi: https://doi.org/10.1016/j.rse.2020.111898.
 
+            Emulator reference:
+            P.G. Brodrick, D.R. Thompson, J.E. Fahlen, M.L. Eastwood, C.M. Sarture, S.R. Lundeen, W. Olson-Duvall,
+            N. Carmon, and R.O. Green. Generalized radiative transfer emulation for imaging spectroscopy reflectance
+            retrievals. Remote Sensing of Environment, 261:112476, 2021.doi: 10.1016/j.rse.2021.112476.
+
 
     Returns:
         np.array
@@ -113,10 +121,11 @@ def main():
     parser.add_argument('--empirical_line', choices=[0,1], type=int, default=0)
     parser.add_argument('--ray_temp_dir', type=str, default='/tmp/ray')
     parser.add_argument('--emulator_base', type=str, default=None)
+    parser.add_argument('--segmentation_size', type=int, default=40)
 
-    args = parser.parse_args()
+    args = parser.parse_args(rawargs)
 
-    if args.sensor not in ['ang', 'avcl', 'neon', 'prism', 'emit']:
+    if args.sensor not in ['ang', 'avcl', 'neon', 'prism', 'emit', 'hyp']:
         if args.sensor[:3] != 'NA-':
             raise ValueError('argument sensor: invalid choice: "NA-test" (choose from '
                              '"ang", "avcl", "neon", "prism", "emit", "NA-*")')
@@ -131,6 +140,23 @@ def main():
     else:
         logging.basicConfig(format='%(message)s', level=args.logging_level, filename=args.log_file)
 
+    rdn_dataset = gdal.Open(args.input_radiance, gdal.GA_ReadOnly)
+    rdn_size = (rdn_dataset.RasterXSize, rdn_dataset.RasterYSize)
+    del rdn_dataset
+    for infile_name, infile in zip(['input_radiance','input_loc','input_obs'],
+                                   [args.input_radiance, args.input_loc, args.input_obs]):
+        if os.path.isfile(infile) is False:
+            err_str = f'Input argument {infile_name} give as: {infile}.  File not found on system.'
+            raise ValueError('argument ' + err_str)
+        if infile_name != 'input_radiance':
+            input_dataset = gdal.Open(infile, gdal.GA_ReadOnly)
+            input_size = (input_dataset.RasterXSize, input_dataset.RasterYSize)
+            if not (input_size[0] == rdn_size[0] and input_size[1] == rdn_size[1]):
+                err_str = f'Input file: {infile_name} size is {input_size}, which does not match input_radiance size: {rdn_size}'
+                raise ValueError(err_str)
+
+
+
     lut_params = LUTConfig(args.lut_config_file)
     if args.emulator_base is not None:
         lut_params.aot_550_range = lut_params.aerosol_2_range
@@ -158,7 +184,9 @@ def main():
         dt = datetime.strptime(paths.fid[:19], 'emit%Y%m%dt%H%M%S')
     elif args.sensor[:3] == 'NA-':
         dt = datetime.strptime(args.sensor[3:], '%Y%m%d')
-
+    elif args.sensor == 'hyp':
+        dt = datetime.strptime(paths.fid[10:17], '%Y%j')
+        
     dayofyear = dt.timetuple().tm_yday
 
     h_m_s, day_increment, mean_path_km, mean_to_sensor_azimuth, mean_to_sensor_zenith, valid, \
@@ -173,7 +201,7 @@ def main():
     if args.wavelength_path:
         chn, wl, fwhm = np.loadtxt(args.wavelength_path).T
     else:
-        radiance_dataset = envi.open(paths.radiance_working_path + '.hdr')
+        radiance_dataset = envi.open(envi_header(paths.radiance_working_path))
         wl = np.array([float(w) for w in radiance_dataset.metadata['wavelength']])
         if 'fwhm' in radiance_dataset.metadata:
             fwhm = np.array([float(f) for f in radiance_dataset.metadata['fwhm']])
@@ -214,6 +242,11 @@ def main():
     logging.info(f'To-sensor Azimuth (deg): {mean_to_sensor_azimuth}')
     logging.info(f'Altitude (km): {mean_altitude_km}')
 
+    if args.emulator_base is not None and mean_altitude_km > 99:
+        logging.info('Adjusting altitude to 99 km for integration with 6S, because emulator is chosen.')
+        mean_altitude_km = 99
+
+
     # We will use the model discrepancy with covariance OR uncorrelated 
     # Calibration error, but not both.
     if args.model_discrepancy_path is not None:
@@ -226,8 +259,8 @@ def main():
         if not exists(paths.lbl_working_path) or not exists(paths.radiance_working_path):
             logging.info('Segmenting...')
             segment(spectra=(paths.radiance_working_path, paths.lbl_working_path),
-                    nodata_value=-9999, npca=5, segsize=SEGMENTATION_SIZE, nchunk=CHUNKSIZE,
-                    n_cores=args.n_cores)
+                    nodata_value=-9999, npca=5, segsize=args.segmentation_size, nchunk=CHUNKSIZE,
+                    n_cores=args.n_cores, loglevel=args.logging_level, logfile=args.log_file)
 
         # Extract input data per segment
         for inp, outp in [(paths.radiance_working_path, paths.rdn_subs_path),
@@ -235,8 +268,9 @@ def main():
                           (paths.loc_working_path, paths.loc_subs_path)]:
             if not exists(outp):
                 logging.info('Extracting ' + outp)
-                extractions(inputfile=inp, labels=paths.lbl_working_path,
-                            output=outp, chunksize=CHUNKSIZE, flag=-9999, n_cores=args.n_cores)
+                extractions(inputfile=inp, labels=paths.lbl_working_path, output=outp,
+                            chunksize=CHUNKSIZE, flag=-9999, n_cores=args.n_cores,
+                            loglevel=args.logging_level, logfile=args.log_file)
 
     if args.presolve == 1:
 
@@ -253,7 +287,7 @@ def main():
             max_water = 6
 
         # run H2O grid as necessary
-        if not exists(paths.h2o_subs_path + '.hdr') or not exists(paths.h2o_subs_path):
+        if not exists(envi_header(paths.h2o_subs_path)) or not exists(paths.h2o_subs_path):
             # Write the presolve connfiguration file
             h2o_grid = np.linspace(0.01, max_water - 0.01, 10).round(2)
             logging.info(f'Pre-solve H2O grid: {h2o_grid}')
@@ -268,14 +302,15 @@ def main():
             del retrieval_h2o
 
             # clean up unneeded storage
-            for to_rm in ['*r_k', '*t_k', '*tp7', '*wrn', '*psc', '*plt', '*7sc', '*acd']:
-                cmd = 'rm ' + join(paths.lut_h2o_directory, to_rm)
-                logging.info(cmd)
-                os.system(cmd)
+            if args.emulator_base is None:
+                for to_rm in ['*r_k', '*t_k', '*tp7', '*wrn', '*psc', '*plt', '*7sc', '*acd']:
+                    cmd = 'rm ' + join(paths.lut_h2o_directory, to_rm)
+                    logging.info(cmd)
+                    os.system(cmd)
         else:
             logging.info('Existing h2o-presolve solutions found, using those.')
 
-        h2o = envi.open(paths.h2o_subs_path + '.hdr')
+        h2o = envi.open(envi_header(paths.h2o_subs_path))
         h2o_est = h2o.read_band(-1)[:].flatten()
 
         p05 = np.percentile(h2o_est[h2o_est > lut_params.h2o_min], 5)
@@ -307,7 +342,8 @@ def main():
         build_main_config(paths, lut_params, h2o_lut_grid, elevation_lut_grid, to_sensor_azimuth_lut_grid,
                           to_sensor_zenith_lut_grid, mean_latitude, mean_longitude, dt, 
                           args.empirical_line == 1, args.n_cores, args.surface_category,
-                          args.emulator_base, uncorrelated_radiometric_uncertainty, args.multiple_restarts)
+                          args.emulator_base, uncorrelated_radiometric_uncertainty, args.multiple_restarts,
+                          args.segmentation_size)
 
         # Run modtran retrieval
         logging.info('Running ISOFIT with full LUT')
@@ -316,14 +352,18 @@ def main():
         del retrieval_full
 
         # clean up unneeded storage
-        for to_rm in ['*r_k', '*t_k', '*tp7', '*wrn', '*psc', '*plt', '*7sc', '*acd']:
-            cmd = 'rm ' + join(paths.lut_modtran_directory, to_rm)
-            logging.info(cmd)
-            os.system(cmd)
+        if args.emulator_base is None:
+            for to_rm in ['*r_k', '*t_k', '*tp7', '*wrn', '*psc', '*plt', '*7sc', '*acd']:
+                cmd = 'rm ' + join(paths.lut_modtran_directory, to_rm)
+                logging.info(cmd)
+                os.system(cmd)
 
     if not exists(paths.rfl_working_path) or not exists(paths.uncert_working_path):
         # Empirical line
         logging.info('Empirical line inference')
+        # Determine the number of neighbors to use.  Provides backwards stability and works
+        # well with defaults, but is arbitrary
+        nneighbors = int(round(3950 / 9 - 35/36 * args.segmentation_size))
         empirical_line(reference_radiance_file=paths.rdn_subs_path,
                        reference_reflectance_file=paths.rfl_subs_path,
                        reference_uncertainty_file=paths.uncert_subs_path,
@@ -333,7 +373,8 @@ def main():
                        input_locations_file=paths.loc_working_path,
                        output_reflectance_file=paths.rfl_working_path,
                        output_uncertainty_file=paths.uncert_working_path,
-                       isofit_config=paths.modtran_config_path)
+                       isofit_config=paths.modtran_config_path,
+                       nneighbors=nneighbors)
 
     logging.info('Done.')
 
@@ -361,8 +402,10 @@ class Pathnames():
             self.fid = split(args.input_radiance)[-1][:21]
         elif args.sensor == 'emit':
             self.fid = split(args.input_radiance)[-1][:19]
-        elif args.sensor[3:] == 'NA-':
+        elif args.sensor[:3] == 'NA-':
             self.fid = os.path.splitext(os.path.basename(args.input_radiance))[0]
+        elif args.sensor == 'hyp': 
+            self.fid = split(args.input_radiance)[-1][:22]
 
         # Names from inputs
         self.aerosol_climatology = args.aerosol_climatology_path
@@ -443,8 +486,11 @@ class Pathnames():
 
         self.sixs_path = os.getenv('SIXS_DIR')
 
-        # isofit file should live at isofit/isofit/core/isofit.py
-        self.isofit_path = os.path.dirname(os.path.dirname(os.path.dirname(isofit.__file__)))
+        if os.getenv('ISOFIT_DIR'):
+            self.isofit_path = os.getenv('ISOFIT_DIR')
+        else:
+             # isofit file should live at isofit/isofit/core/isofit.py
+            self.isofit_path = os.path.dirname(os.path.dirname(os.path.dirname(isofit.__file__)))
 
         if args.sensor == 'ang':
             self.noise_path = join(self.isofit_path, 'data', 'avirisng_noise.txt')
@@ -493,7 +539,7 @@ class Pathnames():
                 logging.info('Staging %s to %s' % (src, dst))
                 copyfile(src, dst)
                 if hasheader:
-                    copyfile(src + '.hdr', dst + '.hdr')
+                    copyfile(envi_header(src), envi_header(dst))
 
 
 class SerialEncoder(json.JSONEncoder):
@@ -544,8 +590,8 @@ class LUTConfig:
         self.h2o_range = [0.05, 5]
 
         # Units of degrees
-        self.to_sensor_azimuth_spacing = 0
-        self.to_sensor_azimuth_spacing_min = 0
+        self.to_sensor_azimuth_spacing = 60
+        self.to_sensor_azimuth_spacing_min = 60
 
         # Units of degrees
         self.to_sensor_zenith_spacing = 10
@@ -665,6 +711,11 @@ class LUTConfig:
             if spatial_data.shape[0]  == 1:
                 spatial_data = np.vstack([spatial_data, spatial_data])
 
+            # Protect memory against huge images
+            if spatial_data.shape[0] > 1e6:
+                 use = np.linspace(0,spatial_data.shape[0]-1,int(1e6),dtype=int)
+                 spatial_data = spatial_data[use,:]
+
             gmm.fit(spatial_data)
             central_angles = np.degrees(np.arctan2(gmm.means_[:, 1], gmm.means_[:, 0]))
             if num_points == 1:
@@ -973,19 +1024,25 @@ def get_metadata_from_loc(loc_file: str, lut_params: LUTConfig, trim_lines: int
 
 def build_presolve_config(paths: Pathnames, h2o_lut_grid: np.array, n_cores: int=-1,
         use_emp_line:bool = False, surface_category="multicomponent_surface",
-        emulator_base: str = None, uncorrelated_radiometric_uncertainty: float = 0.0):
+        emulator_base: str = None, uncorrelated_radiometric_uncertainty: float = 0.0,
+        segmentation_size: int = 400):
     """ Write an isofit config file for a presolve, with limited info.
 
     Args:
         paths: object containing references to all relevant file locations
         h2o_lut_grid: the water vapor look up table grid isofit should use for this solve
         n_cores: number of cores to use in processing
+        use_emp_line: flag whether or not to set up for the empirical line estimation
+        surface_category: type of surface to use
+        emulator_base: the basename of the emulator, if used
+        uncorrelated_radiometric_uncertainty: uncorrelated radiometric uncertainty parameter for isofit
+        segmentation_size: image segmentation size if empirical line is used
     """
 
     # Determine number of spectra included in each retrieval.  If we are
     # operating on segments, this will average down instrument noise
     if use_emp_line:
-        spectra_per_inversion = SEGMENTATION_SIZE
+        spectra_per_inversion = segmentation_size
     else: 
         spectra_per_inversion = 1 
 
@@ -993,7 +1050,7 @@ def build_presolve_config(paths: Pathnames, h2o_lut_grid: np.array, n_cores: int
     if emulator_base is None:
         engine_name = 'modtran'
     else:
-        engine_name = 'simulated_modtran'
+        engine_name = 'sRTMnet'
 
     radiative_transfer_config = {
             "radiative_transfer_engines": {
@@ -1089,7 +1146,8 @@ def build_main_config(paths: Pathnames, lut_params: LUTConfig, h2o_lut_grid: np.
                       to_sensor_zenith_lut_grid: np.array = None, mean_latitude: float = None,
                       mean_longitude: float = None, dt: datetime = None, use_emp_line: bool = True, 
                       n_cores: int = -1, surface_category='multicomponent_surface',
-                      emulator_base: str = None, uncorrelated_radiometric_uncertainty: float = 0.0, multiple_restarts: bool = False):
+                      emulator_base: str = None, uncorrelated_radiometric_uncertainty: float = 0.0,
+                      multiple_restarts: bool = False, segmentation_size=400):
     """ Write an isofit config file for the main solve, using the specified pathnames and all given info
 
     Args:
@@ -1107,20 +1165,20 @@ def build_main_config(paths: Pathnames, lut_params: LUTConfig, h2o_lut_grid: np.
         surface_category: type of surface to use
         emulator_base: the basename of the emulator, if used
         uncorrelated_radiometric_uncertainty: uncorrelated radiometric uncertainty parameter for isofit
-
+        segmentation_size: image segmentation size if empirical line is used
     """
 
     # Determine number of spectra included in each retrieval.  If we are
     # operating on segments, this will average down instrument noise
     if use_emp_line:
-        spectra_per_inversion = SEGMENTATION_SIZE
+        spectra_per_inversion = segmentation_size
     else: 
         spectra_per_inversion = 1 
 
     if emulator_base is None:
         engine_name = 'modtran'
     else:
-        engine_name = 'simulated_modtran'
+        engine_name = 'sRTMnet'
     radiative_transfer_config = {
 
             "radiative_transfer_engines": {
@@ -1344,6 +1402,5 @@ def write_modtran_template(atmosphere_type: str, fid: str, altitude_km: float, d
     with open(output_file, 'w') as fout:
         fout.write(json.dumps(h2o_template, cls=SerialEncoder, indent=4, sort_keys=True))
 
-
 if __name__ == "__main__":
     main()
